from abc import ABC, abstractmethod
from typing import Awaitable, Generic, TypeVar

from .execution_context import FExecutionContext

TData = TypeVar('TData')

class FChannelPublisher(ABC, Generic[TData]):
    @abstractmethod
    def publish(executionContext: FExecutionContext, data: TData) -> Awaitable[None]:
        pass


from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Awaitable, Callable, Generic, TypeVar, Union

from .channel_event import FChannelEvent, TChannelEventData
from .execution_context import FExecutionContext
from .exception import FException

TChannelConsumerCallbackEvent = TypeVar("TEvent", bound="FChannelEvent[TChannelEventData]")

FChannelConsumerCallback = Callable[[FExecutionContext, Union[TChannelConsumerCallbackEvent, FException]], Awaitable[None]]

class FChannelConsumer(ABC, Generic[TChannelEventData, TChannelConsumerCallbackEvent]):
    @abstractmethod
    def add_handler(self, cb: FChannelConsumerCallback[TChannelConsumerCallbackEvent[TChannelEventData]]) -> None:
        pass
    
    @abstractmethod
    def remove_handler(self, cb: FChannelConsumerCallback[TChannelConsumerCallbackEvent[TChannelEventData]]) -> None:
        pass



from abc import ABC, abstractmethod
from typing import Generic, TypeVar

TChannelEventData = TypeVar("TData")

class FChannelEvent(ABC, Generic[TChannelEventData]):
    @property
    @abstractmethod
    def data(self) -> TChannelEventData:
        pass



from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Awaitable, Callable, Generic, TypeVar

from .channel_event import FChannelEvent, TChannelEventData
from .execution_context import FExecutionContext

TChannelConsumerUnbreakableCallbackEvent = TypeVar("TEvent", bound="FChannelEvent[TChannelEventData]")

FChannelConsumerUnbreakableCallback = Callable[[FExecutionContext, TChannelConsumerUnbreakableCallbackEvent], Awaitable[None]]

class FChannelConsumerUnbreakable(ABC, Generic[TChannelEventData, TChannelConsumerUnbreakableCallbackEvent]):
    @abstractmethod
    def add_handler(self, cb: FChannelConsumerUnbreakableCallback[TChannelConsumerUnbreakableCallbackEvent[TChannelEventData]]) -> None:
        pass
    
    @abstractmethod
    def remove_handler(self, cb: FChannelConsumerUnbreakableCallback[TChannelConsumerUnbreakableCallbackEvent[TChannelEventData]]) -> None:
        pass
